//////////////////////////////////////////////////////////////////////////////
//
// 		  Copyright (C) 1996,1997  Matthew Doar  doar@pobox.com
// 
// Permission to use, copy, and distribute this software and its documentation 
// for any purpose with or without fee is hereby granted, provided that the 
// above copyright notice appears with all copies and that both that copyright 
// notice and this permission notice appear in supporting documentation. 
// 
// Permission to modify the software is granted, but not the right to 
// distribute the modified code. Modifications are to be distributed as 
// patches to the released version. 
// 
// This software is provided "as is" without express or implied warranty. 
//
//////////////////////////////////////////////////////////////////////////////

// tiers_pnni.hh

//                             ____  _   _ _   _ ___ 
//                            |  _ \| \ | | \ | |_ _|
//                            | |_) |  \| |  \| || | 
//                            |  __/| |\  | |\  || | 
//                            |_|   |_| \_|_| \_|___|
//                                                   
//
// The basic idea for generating PNNI peer groups is that the PnniModel
// inherits most of its behaviour from the standard Tiers model (Model).
// Each WAN node has a MAN (peer group) associated with it in a 1:1 fashion,
// with MAN i connected to WAN node i. LANs are unchanged in their definition
// The major changes are when outputting the model. Each WAN node is output 
// with its MAN nodes. 
// Currently, all the edges are output as with tiers. This means that there 
// may be edges output as being intra peer group which are actually inter 
// peer group (level1). This does not affect the edge details, just the 
// classification of each edge. A better model would be:
//
// All the WAN edges are output first, then all the MAN-WAN internetwork edges
// which have a WAN node which is not the WAN node in this peer group, then all
// other edges. 
//
// The implementation is that using an optional command line parameter, a
// class PnniModel is created, which inherits mosts of its behaviour from 
// Model, the standard tiers model. Behaviour which is different is all 
// included in this file, and called as virtual functions.
//
//	First version of PNNI network generation model: 
//		September 1997, Matthew Doar
//
// Other ideas for PNNI work: 
// 	(1) Document as:
// 	WAN = 1 peer group, WAN+MANs = multiple peer groups
//  WAN+MANs_LANs = multiple peer groups, two levels of hierarchy
//
//	(2) It would be useful in gnuplot to plot an oval around each peer group
//	and to colour different kinds of edges differently for PNNI
// 
//  (3) Much of this is redundant code

#ifndef _TIERS_PNNI_HH
#define _TIERS_PNNI_HH


////////////////////////////////////////////////////////////////////////
// PNNI Model
//
// The major class for the PNNI network model generation code
////////////////////////////////////////////////////////////////////////
class Model;
class PnniModel : public Model
{
public:
  PnniModel(void);
  ~PnniModel(void);

private:
  // Functions
  bool ConnectMans();

  bool OutputParams();
  bool OutputNodes();
  bool OutputEdges();

  bool CreateGlobalNodeList(Node* &nodes);
  bool OutputEdgesGnuplot();

};

#endif // _TIERS_PNNI_HH

// end of file


